/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit;

import java.io.File;
import java.util.logging.Handler;

import javax.jbi.management.DeploymentException;
import javax.xml.namespace.QName;

import org.junit.rules.TestRule;
import org.ow2.petals.component.framework.junit.impl.ServiceConfiguration;
import org.ow2.petals.component.framework.junit.rule.ComponentUnderTest;
import org.ow2.petals.component.framework.junit.rule.NativeServiceConfigurationFactory;
import org.ow2.petals.component.framework.junit.rule.ServiceConfigurationFactory;

/**
 * @author Marc Jambert - EBM WebSourcing
 */
public interface Component extends TestRule {

    public static final long DEFAULT_TIMEOUT = 30000;

    /**
     * Property set on messages that were sent with sendSync from the component under test
     */
    public static final String SENDSYNC_EXCHANGE_PROPERTY = "org.ow2.petals.component.framework.junit.Component.SendSync";

    public String getComponentName();

    /**
     * Check that the component instance under test is installed
     * 
     * @return <code>true</code> if the component instance is installed, <code>false</code> otherwise.
     */
    public boolean isInstalled();

    /**
     * Start the component instance
     */
    public void start();

    /**
     * Check that the component instance under test is started
     * 
     * @return <code>true</code> if the component instance is started, <code>false</code> otherwise.
     */
    public boolean isStarted();

    /**
     * Stop the component.
     */
    public void stop();

    /**
     * Shutdown the component. All SU deployed are automatically undeployed during before the shutdown.
     */
    public void shutdown();
    
    /**
     * Deploy a service, as service unit, into the component
     * 
     * @param suName
     *            The service configuration name
     * @param serviceConfiguration
     *            The service configuration
     * @throws DeploymentException
     *             The deployment fails
     */
    public void deployService(final String suName, final ServiceConfiguration serviceConfiguration)
            throws DeploymentException;

    /**
     * Undeploy a service as a service unit
     * 
     * @param serviceConfigurationId
     *            The service configuration identifier to undeploy
     */
    public void undeployService(final String suName);
    
    /**
     * Check that a service, as service unit, is deployed
     * 
     * @param suName
     *            The service to check
     * @return <code>true</code> if the service is deployed, <code>false</code> otherwise.
     */
    public boolean isServiceDeployed(final String suName);
    
    /**
     * Undeploy all services
     */
    public void undeployAllServices();

    /**
     * Retrieve the installation directory of a service configuration (ie. service unit)
     * 
     * @param suName
     *            The service configuration name
     * @return The installation directory of a service configuration
     */
    public File getServiceConfigurationInstallDir(final String suName);

    /**
     * <p>
     * As a service consumer, send a message exchange in a <b>asynchronous</b> way to the JBI listeners of the component
     * under test. Use {@link #pollResponseFromProvider()} to get the reply of an IN-OUT exchange.
     * </p>
     * 
     * @param message
     *            The message exchange to send
     */
    public void pushRequestToProvider(final RequestMessage message);

    /**
     * <p>
     * As a service consumer, send a message exchange in a <b>asynchronous</b> way to the JBI listeners of the component
     * under test.
     * </p>
     * 
     * @param message
     *            The message exchange to send
     */
    public void pushStatusToProvider(final ResponseMessage message);

    /**
     * <p>
     * As a service provider, send a message exchange in an <b>asynchronous</b> way to the JBI listeners of the
     * component under test.
     * </p>
     * <p>
     * Note: An end or failure MONIT trace is automatically logged according to the content of the response message.
     * </p>
     * 
     * @param message
     *            The message exchange to send
     */
    public void pushResponseToConsumer(final ResponseMessage message);
    
    /**
     * <p>
     * As a service consumer, wait a message exchange, as <b>reply</b>, coming from the component under test using the
     * default timeout {@link #DEFAULT_TIMEOUT}
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the response of a IN-OUT service request
     * previously send with {@link #pushRequestToProvider(RequestMessage)}
     * </p>
     * 
     * @return The message exchange returned by the service provider
     */
    public ResponseMessage pollResponseFromProvider();

    /**
     * <p>
     * As a service consumer, wait a message exchange coming from the component under test.
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the response of a service request previously
     * send with {@link #pushRequestToProvider(RequestMessage)}
     * </p>
     * 
     * @param timeout
     *            Max time to wait in milliseconds. If <code>0</code>, no timeout will occur.
     * @return The message exchange returned by the service provider
     */
    public ResponseMessage pollResponseFromProvider(final long timeout);

    /**
     * <p>
     * As a service consumer, wait a message exchange, as <b>status</b>, coming from the component under test using the
     * default timeout {@link #DEFAULT_TIMEOUT}
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the status of an IN-ONLY service request
     * previously send with {@link #pushRequestToProvider(RequestMessage)}
     * </p>
     * 
     * @return The message exchange returned by the service provider
     */
    public ResponseMessage pollStatusFromProvider();

    /**
     * <p>
     * As a service consumer, wait a message exchange, as <b>status</b>, coming from the component under test.
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the status of an IN-ONLY service request
     * previously send with {@link #pushRequestToProvider(RequestMessage)}
     * </p>
     * 
     * @param timeout
     *            Max time to wait in milliseconds. If <code>0</code>, no timeout will occur.
     * @return The message exchange returned by the service provider
     */
    public ResponseMessage pollStatusFromProvider(final long timeout);

    /**
     * <p>
     * As a service provider, wait for a message exchange, as <b>status</b>, coming from the component under test using
     * the default timeout {@link #DEFAULT_TIMEOUT}
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the status of an IN-ONLY service request
     * previously send with {@link #pushResponseToConsumer(ResponseMessage)}
     * </p>
     * 
     * @return The message exchange returned by the service consumer
     */
    public ResponseMessage pollStatusFromConsumer();

    /**
     * <p>
     * As a service provider, wait for a message exchange, as <b>status</b>, coming from the component under test using
     * the default timeout {@link #DEFAULT_TIMEOUT}
     * </p>
     * <p>
     * The goal of this method is to get, in an <b>asynchronous</b> way, the status of an IN-ONLY service request
     * previously send with {@link #pushResponseToConsumer(ResponseMessage)}
     * 
     * @param timeout
     *            Max time to wait in milliseconds. If <code>0</code>, no timeout will occur.
     * @return The message exchange returned by the service consumer
     */
    public ResponseMessage pollStatusFromConsumer(final long timeout);
    
    /**
     * Clear the queue of responses from other services to the component under tests.
     * 
     * Can be used for example inbetween tests to be sure there is no left-overs from failing tests.
     */
    void clearResponsesFromProvider();

    /**
     * Get the number of responses (from other services to the component under tests) still in the queue.
     */
    int getResponsesFromProviderCount();

    /**
     * <p>
     * As a service provider, wait a message exchange coming from the component under test using the default timeout
     * {@link #DEFAULT_TIMEOUT}. Use {@link #pushResponseToConsumer(ResponseMessage)} to answer it.
     * 
     * </p>
     * <p>
     * Note: A beginning MONIT trace is automatically logged according to the content of the received incoming message.
     * </p>
     *
     * If the message was sent by the component under test using sendSync, the property of
     * {@link #SENDSYNC_EXCHANGE_PROPERTY} ({@value #SENDSYNC_EXCHANGE_PROPERTY}) will be set on the message exchange.
     * 
     * @return The message exchange sent by the component under test
     */
    public RequestMessage pollRequestFromConsumer();

    /**
     * <p>
     * As a service provider, wait a message exchange coming from the component under test. Use
     * {@link #pushResponseToConsumer(ResponseMessage)} to answer it.
     * </p>
     * <p>
     * Note: A beginning MONIT trace is automatically logged according to the content of the received incoming message.
     * </p>
     * 
     * If the message was sent by the component under test using sendSync, the property of
     * {@link #SENDSYNC_EXCHANGE_PROPERTY} ({@value #SENDSYNC_EXCHANGE_PROPERTY}) will be set on the message exchange.
     * 
     * @param timeout
     *            Max time to wait in milliseconds. If <code>0</code>, no timeout will occur.
     * @return The message exchange sent by the component under test, or <code>null</code> if a time-out occurs.
     */
    public RequestMessage pollRequestFromConsumer(final long timeout);

    /**
     * Clear the queue of requests from the component under test to other services.
     * 
     * Can be used for example inbetween tests to be sure there is no left-overs from failing tests.
     */
    void clearRequestsFromConsumer();

    /**
     * Get the number of requests (from the component under test to other services) still in the queue.
     */
    int getRequestsFromConsumerCount();

    /**
     * <p>
     * Send a message exchange in a <b>synchronous</b> way to the component under test, and wait for its processing to
     * finish.
     * </p>
     * 
     * @param message
     *            The message exchange to send
     */
    public void pushSynchronouslyRequestToProvider(final RequestMessage message);

    /**
     * Get the native endpoint associated to the given native service.
     * 
     * @param service
     *            The service for which the endpoint name is asked
     * @return The native endpoint name, or <code>null</code> if the given service does not exist
     */
    public String getNativeEndpointName(final QName service);

    /**
     * Add log handler, typically an InMemoryLogHandler
     * 
     * @param logHandler
     * @return
     */
    ComponentUnderTest addLogHandler(Handler logHandler);

    /**
     * @return The base directory where the component is installed, or <code>null</code> if the component is not
     *         instantiated
     */
    File getBaseDirectory();

    /**
     * Register a service as a service unit to deploy when the component will be initialized/started
     * 
     * @param serviceCfgId
     *            The service configuration identifier. It is used to retrieve service configuration when working with
     *            messages.
     * @param serviceConfigurationFactory
     *            The service configuration factory creating the service to deploy
     */
    ComponentUnderTest registerServiceToDeploy(String serviceCfgId,
            ServiceConfigurationFactory serviceConfigurationFactory);

    /**
     * Deploy a service as a service unit
     * 
     * @param serviceCfgId
     *            The service configuration identifier. It is used to retrieve service configuration when working with
     *            messages.
     * @param serviceConfigurationFactory
     *            The service configuration factory creating the service to deploy
     * @throws DeploymentException
     *             The deployment fails
     */
    void deployService(String serviceCfgId, ServiceConfigurationFactory serviceConfigurationFactory)
            throws DeploymentException;

    /**
     * Register a native service to deploy when the component will be initialized/started
     * 
     * @param serviceCfgId
     *            The service configuration identifier. It is used to retrieve service configuration when working with
     *            messages.
     * @param serviceConfigurationFactory
     *            The service configuration factory creating the native service to deploy
     */
    ComponentUnderTest registerNativeServiceToDeploy(String serviceCfgId,
            NativeServiceConfigurationFactory serviceConfigurationFactory);

    /**
     * Register a native service to deploy: the service is not actually deployed as it is already done as part as the
     * component start. This is only useful for knowing it is registered from the tests point of view.
     * 
     * @param serviceCfgId
     *            The service configuration identifier. It is used to retrieve service configuration when working with
     *            messages.
     * @param serviceConfigurationFactory
     *            The service configuration factory creating the native service to deploy
     */
    void registerNativeService(String serviceCfgId, NativeServiceConfigurationFactory serviceConfigurationFactory);

    /**
     * Register an external service provider, not running on the component under test.
     * 
     * @param serviceName
     *            The service name of the external service provider
     * @param endpointName
     *            The endpoint name of the external service provider
     */
    ComponentUnderTest registerExternalServiceProvider(QName serviceName, String endpointName);

    /**
     * Retrieve a service configuration according to a given service configuration identifier
     * 
     * @param suName
     *            The service configuration identifier of the service configuration to retrieve
     * @return The service configuration, or <code>null</code> is not found
     */
    ServiceConfiguration getServiceConfiguration(String suName);

}
